#include "FightScene.h"
#include "GameScene.h"

using namespace cocos2d;
using namespace CocosDenshion;

bool FightScene::init() {
    if (!CCLayer::init())
        return false;
    
   
    return true;
}

void FightScene::setup() {
	setIsTouchEnabled( true );
    
	CCSize screenSize = CCDirector::sharedDirector()->getWinSize();
    CCLayerColor* layer = CCLayerColor::layerWithColorWidthHeight(ccc4f(255, 255, 0, 255), screenSize.width, screenSize.height);
    addChild(layer);
    
	CCLabelTTF *label = CCLabelTTF::labelWithString("[flee]", "Futura-CondensedExtraBold", 32);
    label->setColor( ccc3(200,0,200) );
    
    CCMenuItem *button = CCMenuItemLabel::itemWithLabel(label, this, menu_selector(FightScene::onBack));
    
    CCMenu *menu = CCMenu::menuWithItem(button);
    
    menu->setPosition(50, screenSize.height-25);
    
    addChild(menu);
    
    
    
    //    CCPoint centerOfView = ccp(0, 0);
    //    setPosition(centerOfView);
    
    //
    mTileSize = 50;
    mWidth = 8;
    mHeight = 6;
    mTiles = new Tile[mWidth*mHeight];
    
    mMatchSize = 0;
    mCurPickX = mCurPickY = -1;
    mAnimating = 0;
    
    mbSkipEffects = true;
    mCurPlayer = -1;
    
    // TODO: set externally
    mFightData.bIsValid = true;
    //    mFightData.mMaxHealth[0] = 25;
    //    mFightData.mMaxHealth[1] = 25;
    //    mFightData.mExperience[0] = mFightData.mExperience[1] = 0;
    //    mFightData.mCash[0] = mFightData.mCash[1] = 0;
    
    //
    mFightData.mHealth[0] = mFightData.mMaxHealth[0];
    mFightData.mHealth[1] = mFightData.mMaxHealth[1];
    
    // add your avatar
    mAvatar[0] = CCSprite::spriteWithSpriteFrameName("bunny2");
    mAvatar[0]->setAnchorPoint(CCPoint(0.0f, 0.0f));
    mAvatar[0]->setScale(8.0f);
    mAvatar[0]->setPosition( CCPoint(40.0f, screenSize.height*0.66) );
    addChild(mAvatar[0]);
    
    mAvatarHealth[0] = CCSprite::spriteWithSpriteFrameName("healthbar");
    //    mAvatarHealth[0]->setAnchorPoint(CCPoint(0.5f, 0.5f));
    mAvatarHealth[0]->setPositionInPixels( CCPoint(113, screenSize.height-290) );
    mAvatarHealth[0]->setScaleY(4.0f);
    addChild(mAvatarHealth[0]);
    //
    mAvatarHealth[1] = CCSprite::spriteWithSpriteFrameName("healthbar");
    ////    mAvatarHealth[1]->setAnchorPoint(CCPoint(0.5f, 0.5f));
    mAvatarHealth[1]->setPositionInPixels( CCPoint(851, screenSize.height-415) );
    mAvatarHealth[1]->setScaleY(4.0f);
    addChild(mAvatarHealth[1]);
    
    updateHealthBars();
    
    // and your oponents avatar
    mAvatar[1] = CCSprite::spriteWithSpriteFrameName("building1");
    mAvatar[1]->setAnchorPoint(CCPoint(0.66f, 0.0f));
    mAvatar[1]->setScale(8.0f);
    mAvatar[1]->setPosition( CCPoint(screenSize.width, screenSize.height*0.5) );
    addChild(mAvatar[1]);
    
    setupTiles();
    
    CCTouchDispatcher::sharedDispatcher()->addTargetedDelegate(this, kCCMenuTouchPriority+1, true);
}

void FightScene::updateHealthBars() {
    for (int i=0; i<2; ++i) {
        float percent = mFightData.mHealth[i]/(float)mFightData.mMaxHealth[i];
        mAvatarHealth[i]->setScaleX(percent*100.0f);
    }
}

void FightScene::cleanup() {
    CCLayer::cleanup();
    delete[] mTiles;
}

CCPoint FightScene::tileCoordForPoint(CCPoint p) {
	CCSize screenSize = CCDirector::sharedDirector()->getWinSize();
    int ox = (screenSize.width-mTileSize*mWidth)*0.5f;
    int oy = (screenSize.height-mTileSize*mHeight)*0.5f;
    return CCPoint((p.x-ox+mTileSize*0.5f)/mTileSize, (p.y-oy+mTileSize*0.5f)/mTileSize); // need half tile width offset as tiles are drawn centered at pt
}

CCPoint FightScene::pointCoordForTile(int x, int y) {
	CCSize screenSize = CCDirector::sharedDirector()->getWinSize();
    int ox = (screenSize.width-mTileSize*mWidth)*0.5f;
    int oy = (screenSize.height-mTileSize*mHeight)*0.5f;
    return CCPoint(x*mTileSize+ox, y*mTileSize+oy);
}

FightScene::Tile* FightScene::getTile(int x, int y) {
    if (x < 0 || x >= mWidth
        || y < 0 || y >= mHeight)
        return NULL;
    
    return &mTiles[x+y*mWidth];
}

bool FightScene::canCollapse() {
    for (int col=0; col<mWidth; ++col) {
        for (int row=0; row<mHeight; ++row) {
            if (getTile(col, row)->type == Tile_Empty)
                return true;
        }
    }
    
    return false;
}

bool FightScene::canMakeMove() {
    for (int col=0; col<mWidth-1; ++col) {
        for (int row=0; row<mHeight-1; ++row) {
            if (canSwap(col, row, col+1, row) || canSwap(col, row, col, row+1))
                return true;
        }
    }
    return false;
}

void FightScene::doAITurn() {
    // FIXME: TODO: get a list of all possible moves, have a high random chance to do an attack one if its there
    // if not, do a random other one
    for (int col=0; col<mWidth-1; ++col) {
        for (int row=0; row<mHeight-1; ++row) {
            if (canSwap(col, row, col+1, row)) {
                doSwap(col, row, col+1, row);
                return;
            }
            
            if (canSwap(col, row, col, row+1)) {
                doSwap(col, row, col, row+1);
                return;
            }
        }
    }
    CCAssert(false, "unable to play AI turn, no move found");
}

void FightScene::checkEndGame() {
}


void FightScene::flashTextCentered(const char* str, int fontSize, const ccColor3B& color3, float travel, float hold) {
    CCSize screenSize = CCDirector::sharedDirector()->getWinSize();
    CCLabelTTF *label = CCLabelTTF::labelWithString(str, "Futura-CondensedExtraBold", fontSize);
    label->setOpacity(0);
    label->setColor( color3 );
    label->setPosition( CCPoint(screenSize.width*0.5f, screenSize.height*1.5f) );
    addChild(label);
    CCFiniteTimeAction* delay = CCDelayTime::actionWithDuration(1.0f);
    CCFiniteTimeAction* appear = CCFadeIn::actionWithDuration(0.1f);
    CCFiniteTimeAction* moveTo = CCMoveTo::actionWithDuration(travel, CCPoint(screenSize.width*0.5f, screenSize.height*0.5f));
    CCFiniteTimeAction* complete = CCCallFunc::actionWithTarget(label, callfunc_selector(CCNode::removeFromParentAndCleanup));
    CCFiniteTimeAction* blink = CCBlink::actionWithDuration(hold, 16);
    label->runAction(CCSequence::actions(delay, appear, moveTo, blink, complete, NULL));
}

void FightScene::flashText(const char* str, int fontSize, const ccColor3B& color3, float travel, float hold) {
    CCSize screenSize = CCDirector::sharedDirector()->getWinSize();
    CCLabelTTF *label = CCLabelTTF::labelWithString(str, "Futura-CondensedExtraBold", fontSize);
    label->setOpacity(0);
    label->setColor( color3 );
    label->setPosition( CCPoint(screenSize.width*1.5f, screenSize.height) );
    addChild(label);
    CCFiniteTimeAction* delay = CCDelayTime::actionWithDuration(1.0f);
    CCFiniteTimeAction* appear = CCFadeIn::actionWithDuration(0.1f);
    CCFiniteTimeAction* moveTo = CCMoveTo::actionWithDuration(travel, CCPoint(screenSize.width*0.5f, screenSize.height*0.75f));
    CCFiniteTimeAction* complete = CCCallFunc::actionWithTarget(label, callfunc_selector(CCNode::removeFromParentAndCleanup));
    CCFiniteTimeAction* blink = CCBlink::actionWithDuration(hold, 16);
    label->runAction(CCSequence::actions(delay, appear, moveTo, blink, complete, NULL));
}

void FightScene::onGameWin(ccTime dt) {
	unschedule(schedule_selector(FightScene::onGameWin));
    CCTransitionScene* transition = CCTransitionSplitRows::transitionWithDuration(5.0, GameScene::scene(mFightData));                    
    CCDirector::sharedDirector()->replaceScene(transition);
}

void FightScene::onGameLose(ccTime dt) {
	unschedule(schedule_selector(FightScene::onGameLose));
    CCTransitionScene* transition = CCTransitionRotoZoom::transitionWithDuration(1.0, GameScene::scene(mFightData));
    CCDirector::sharedDirector()->replaceScene(transition);
}

void FightScene::onNoMoreMoves(ccTime dt) {
	unschedule(schedule_selector(FightScene::onNoMoreMoves));
//    --mAnimating;
    mCurPlayer = -1;
    destroyMatches(true);    
    setupTiles();
}

void FightScene::beginTurn() {
    if (mAnimating)
        return;
    
    if (!canMakeMove()) {
//        ++mAnimating;
        flashTextCentered("NO MORE MOVES!!!", 80, ccc3(0,0,200), 0.5f, 4.0);
        schedule(schedule_selector(FightScene::onNoMoreMoves), 5.0f);
        return;
    }
    
    if (mCurPlayer == -1) {
        mbSkipEffects = false;
        mCurPlayer = 0;
        // AMG FIRST TURN EVAR
        flashText("BATTLE!!!", 80, ccc3(200,0,0), 0.5f, 3.0);
        // lulz
//        doAITurn();
    } else {
        if (mMatchSize >= 4) {
            flashText("BONUS MOVE!!!", 32, ccc3(0,200,0), 0.1f, 1.0);
        } else {
            mCurPlayer = 1-mCurPlayer;
        }
        mMatchSize = 0;

        // 
        if (mCurPlayer == 0) {
            if (mMatchSize < 4)
                flashText("GO!", 32, ccc3(0,200,0), 0.1f, 1.0);
            
            // lulz
//            doAITurn();
        } else {
            // AIs TURN!
            doAITurn();
        }
    }
    
    mAvatar[mCurPlayer]->setOpacity(255);
    mAvatarHealth[mCurPlayer]->setOpacity(255);
    mAvatar[1-mCurPlayer]->setOpacity(128);
    mAvatarHealth[1-mCurPlayer]->setOpacity(128);
}

// returns true if something moved
bool FightScene::collapse() {
    bool bDidCollapse = false;
    
    for (int col=0; col<mWidth; ++col) {
        // collapse crud as usual
        for (int row=0; row < mHeight-1; ++row) {
            Tile* A = getTile(col, row+1);
            Tile* B = getTile(col, row);
            
            if (A->type != Tile_Empty && B->type == Tile_Empty) {
                // actually collapse
                bDidCollapse = true;
                ++mAnimating;
                Tile tmp = *A;
                *A = *B;
                *B = tmp;
                
                float skip = mbSkipEffects ? 0.0f : 1.0f;
                CCFiniteTimeAction* moveToB = CCMoveTo::actionWithDuration(0.25f*skip, pointCoordForTile(col, row));
                CCFiniteTimeAction* complete = CCCallFunc::actionWithTarget(this, callfunc_selector(FightScene::onCollapseComplete));
                B->sprite->runAction(CCSequence::actions(moveToB, complete, NULL));
            }
        } // row
    } // col

    // fill the top row in if its blank
    for (int col=0; col<mWidth; ++col) {
        Tile* top = getTile(col, mHeight-1);
        if (top->type == Tile_Empty) {
            CCAssert(!top->sprite, "sprite not nulled out");
            top->type = randomTileType();
            top->sprite = spriteForTileType(top->type);
            top->sprite->setScale(0.01f);
            top->sprite->setPosition(pointCoordForTile(col, mHeight));
            addChild(top->sprite);
            
            bDidCollapse = true;
            ++mAnimating;
            
            float skip = mbSkipEffects ? 0.0f : 1.0f;
            CCFiniteTimeAction* scaleTo = CCScaleTo::actionWithDuration(0.125f*skip, 2.0f);
            CCFiniteTimeAction* moveTo = CCMoveTo::actionWithDuration(0.125f*skip, pointCoordForTile(col, mHeight-1));
            CCFiniteTimeAction* complete = CCCallFunc::actionWithTarget(this, callfunc_selector(FightScene::onCollapseComplete));
            top->sprite->runAction(CCSequence::actions(scaleTo, moveTo, complete, NULL));
        }
    }

    // do it
    return bDidCollapse;
}

void FightScene::onCollapseComplete() {
    --mAnimating;
    
    if (mAnimating == 0) {
        if (!collapse()) {
            markTiles();
            destroyMatches();
            collapse();
        }
    }
    
    if (mAnimating == 0) {
        beginTurn();
    }
}

// return true if something was destroyed
bool FightScene::destroyMatches(bool bForceClearBoard) {
    bool bDidDestroy = false;
    ETileType matchType = Tile_Empty;

    int tmpMatchSize = 0;

    // do it
    for (int row=0; row<mHeight; ++row) {
        for (int col=0; col<mWidth; ++col) {
            Tile* tile = getTile(col, row);
            if (tile->mark || bForceClearBoard) {
                if (!mbSkipEffects) {
                    // FIXME: TODO: do some fancy looking stuff
                }
                
                matchType = tile->type;
                tile->type = Tile_Empty;
                removeChild(tile->sprite, true);
                tile->sprite = NULL;
                
                bDidDestroy = true;
                ++tmpMatchSize;
            }
        }
    }
    
    if (tmpMatchSize > mMatchSize)
        mMatchSize = tmpMatchSize;
    
    if (mCurPlayer == 0 || mCurPlayer == 1) {
        if (!mbSkipEffects) {
            switch (matchType) {
                case Tile_Empty:
                    break;
                case Tile_Attack:
                    CocosDenshion::SimpleAudioEngine::sharedEngine()->playEffect("boom.wav");
                    break;
                case Tile_Health:
                    CocosDenshion::SimpleAudioEngine::sharedEngine()->playEffect("match1.wav");
                    break;
                case Tile_Experience:
                    CocosDenshion::SimpleAudioEngine::sharedEngine()->playEffect("match2.wav");
                    break;
                case Tile_Cash:
                    CocosDenshion::SimpleAudioEngine::sharedEngine()->playEffect("match3.wav");
                    break;
                default:
                    CocosDenshion::SimpleAudioEngine::sharedEngine()->playEffect("match_default.wav");
                    break;
            }
        }
        
        // award the appropriate thing (if its not a board reset)
        if (!bForceClearBoard) {
            if (matchType == Tile_Attack) {
                int other = 1-mCurPlayer;
                mFightData.mHealth[other] = max(mFightData.mHealth[other]-4, 0);
                updateHealthBars();

                CCFiniteTimeAction* scaleup = CCScaleTo::actionWithDuration(0.125f, 6.0f);
                CCFiniteTimeAction* tintup = CCTintTo::actionWithDuration(0.125f, 0xFF, 0x00, 0x00);
                CCFiniteTimeAction* tintdown = CCTintTo::actionWithDuration(0.125f, 0xFF, 0xFF, 0xFF);
                CCFiniteTimeAction* scaledown = CCScaleTo::actionWithDuration(0.125f, 8.0f);
                mAvatar[other]->runAction(CCSequence::actions(scaleup, tintup, tintdown, scaledown, NULL));

                if (mFightData.mHealth[other] <= 0) {
                    // hack, lock us in an animating state so we cant do anything
                    ++mAnimating;
                    
                    // TODO: effect?
                    if (mCurPlayer == 0) {
                        // WIN WIN WIN
                        flashTextCentered("VICTORY!!!", 80, ccc3(0,0,200), 2.0f, 8.0);
                        schedule(schedule_selector(FightScene::onGameWin), 8.0f);
                    } else {
                        flashTextCentered("defeat :(", 80, ccc3(0,0,200), 2.0f, 8.0);
                        schedule(schedule_selector(FightScene::onGameLose), 8.0f);
                    }
                }
            } else if (matchType == Tile_Experience) {
                CCFiniteTimeAction* tintup = CCTintTo::actionWithDuration(0.125f, 0x00, 0xFF, 0x00);
                CCFiniteTimeAction* tintdown = CCTintTo::actionWithDuration(0.125f, 0xFF, 0xFF, 0xFF);
                mAvatar[mCurPlayer]->runAction(CCSequence::actions(tintup, tintdown, NULL));
                
                ++mFightData.mExperience[mCurPlayer];
            } else if (matchType == Tile_Cash) {
                CCFiniteTimeAction* tintup = CCTintTo::actionWithDuration(0.125f, 0xFF, 0xFF, 0x00);
                CCFiniteTimeAction* tintdown = CCTintTo::actionWithDuration(0.125f, 0xFF, 0xFF, 0xFF);
                mAvatar[mCurPlayer]->runAction(CCSequence::actions(tintup, tintdown, NULL));
                
                ++mFightData.mCash[mCurPlayer];
            } else if (matchType == Tile_Health) {
                CCFiniteTimeAction* tintup = CCTintTo::actionWithDuration(0.125f, 0xFF, 0x00, 0xFF);
                CCFiniteTimeAction* tintdown = CCTintTo::actionWithDuration(0.125f, 0xFF, 0xFF, 0xFF);
                mAvatar[mCurPlayer]->runAction(CCSequence::actions(tintup, tintdown, NULL));
                
                mFightData.mHealth[mCurPlayer] = min(mFightData.mHealth[mCurPlayer]+1, mFightData.mMaxHealth[mCurPlayer]);
                updateHealthBars();
            }
        }
    }
    
    return bDidDestroy;
}

void FightScene::clearMarks() {
    for (int row=0; row<mHeight; ++row) {
        for (int col=0; col<mWidth; ++col) {
            getTile(col, row)->mark = false;
        }
    }
}

bool FightScene::markTiles() {
    clearMarks();
    
    bool bDidMark = false;
    
    // check for horizontal matches
    for (int row=0; row<mHeight; ++row) {
        for (int col=0; col<mWidth-2; ++col) {
            Tile* A = getTile(col, row);
            Tile* B = getTile(col+1, row);
            Tile* C = getTile(col+2, row);
            if (A->type != Tile_Empty && A->type == B->type && A->type == C->type) {
                A->mark = B->mark = C->mark = true;
                bDidMark = true;
            }
        }
    }
    
    // check for vertical matches
    for (int col=0; col<mWidth; ++col) {
        for (int row=0; row<mHeight-2; ++row) {
            Tile* A = getTile(col, row);
            Tile* B = getTile(col, row+1);
            Tile* C = getTile(col, row+2);
            if (A->type != Tile_Empty && A->type == B->type && A->type == C->type) {
                A->mark = B->mark = C->mark = true;
                bDidMark = true;
            }
        }
    }

    // 
    return bDidMark;
}

bool FightScene::swapHasAffect(Tile* A, Tile* B) {
    Tile tmp;

    // swap
    tmp = *A;
    *A = *B;
    *B = tmp;
    
    // look for matches
    bool result = markTiles();
    
    // unswap
    tmp = *A;
    *A = *B;
    *B = tmp;
    
    return result;
}

void FightScene::doSwap(int x1, int y1, int x2, int y2) {
    Tile* A = getTile(x1, y1);
    Tile* B = getTile(x2, y2);
    
    float skip = mbSkipEffects ? 0.0f : 1.0f;

    CCFiniteTimeAction* moveA = CCMoveTo::actionWithDuration(0.33f*skip, B->sprite->getPosition());
    CCFiniteTimeAction* moveB = CCMoveTo::actionWithDuration(0.33f*skip, A->sprite->getPosition());    
    CCFiniteTimeAction* scaleToA = CCScaleTo::actionWithDuration(0.25f*skip, 2.0f);
    CCFiniteTimeAction* scaleToB = CCScaleTo::actionWithDuration(0.25f*skip, 2.0f);
    CCFiniteTimeAction* completeA = CCCallFunc::actionWithTarget(this, callfunc_selector(FightScene::onSwapComplete));
    CCFiniteTimeAction* completeB = CCCallFunc::actionWithTarget(this, callfunc_selector(FightScene::onSwapComplete));
    
    mAnimating += 2;
    // actually swap teh' data
    Tile tmp = *A;
    *A = *B;
    *B = tmp;
    
    B->sprite->runAction(CCSequence::actions(moveA, scaleToA, completeA, NULL));
    A->sprite->runAction(CCSequence::actions(moveB, scaleToB, completeB, NULL));
}

// return true if something was swapped
bool FightScene::canSwap(int x1, int y1, int x2, int y2) {
    Tile* A = getTile(x1, y1);
    Tile* B = getTile(x2, y2);    
    
    if (!A || !B)
        return false;
    
    // only allow adjoining items to swap
    int dx = abs(x1-x2);
    int dy = abs(y1-y2);
    if (!((dx == 1 && dy == 0) || (dx == 0 && dy == 1)))
        return false;
        
    // only allow swaps that DO something
    if (!swapHasAffect(A, B))
        return false;
    
    return true;
}

void FightScene::onSwapComplete() {
    --mAnimating;
    CCAssert(mAnimating >= 0, "wtfmate");
    
    CCAssert(!canCollapse(), "wtf, swap complete but there was space to collapse");
    
    if (mAnimating == 0) {
        // wahoo, a match!
        if (markTiles()) {
            if (destroyMatches()) {
                collapse();
            }
        }
    }
}

FightScene::ETileType FightScene::randomTileType() {
    return (ETileType)mRandom.randInt((int)Tile_NumTypes);
}

CCSprite* FightScene::spriteForTileType(ETileType type) {
    switch (type) {
        case Tile_R:
            return CCSprite::spriteWithSpriteFrameName("bunny0");
        case Tile_G:
            return CCSprite::spriteWithSpriteFrameName("butterfly");
        case Tile_Cash:
            return CCSprite::spriteWithSpriteFrameName("penny");
        case Tile_Attack:
            return CCSprite::spriteWithSpriteFrameName("cannon");
        case Tile_Experience:
            return CCSprite::spriteWithSpriteFrameName("star");
        case Tile_Health:
            return CCSprite::spriteWithSpriteFrameName("heart");
        default:
            CCAssert(false, "wtf, invalid tile type");
            break;
    }
    
    return NULL;
}

void FightScene::setupTiles() {
    CCSpriteFrameCache::sharedSpriteFrameCache()->addSpriteFramesWithFile("sprites.plist");
    CCSpriteBatchNode *spriteSheet = CCSpriteBatchNode::batchNodeWithFile("sprites.pvr.gz");
    spriteSheet->getTexture()->setAliasTexParameters(); // pixelate instead of blur
    addChild(spriteSheet);
    
    for (int row=0; row<mHeight; ++row) {
        for (int col=0; col<mWidth; ++col) {
            getTile(col, row)->type = randomTileType();
        }
    }
    
    for (int row=0; row<mHeight; ++row) {
        for (int col=0; col<mWidth; ++col) {
            Tile* tile = getTile(col, row);
            CCAssert(tile, "");
            
            tile->sprite = spriteForTileType(tile->type);
            tile->sprite->setScale(2.0f);

            tile->sprite->setPosition(pointCoordForTile(col, row));
            addChild(tile->sprite);
        }
    }

    // keep simulating until we come to rest
    if (markTiles()) {
        if (destroyMatches())
            collapse();
    }
    if (mAnimating == 0) {
        beginTurn();
    }
}

void FightScene::onBack() {
    CocosDenshion::SimpleAudioEngine::sharedEngine()->playBackgroundMusic("wolfram.dance.1585.1191346988.mp3", true);
    CCTransitionScene* transition = CCTransitionRotoZoom::transitionWithDuration(1.0, GameScene::scene(mFightData));
    CCDirector::sharedDirector()->replaceScene(transition);
}




bool FightScene::ccTouchBegan(CCTouch *touch, CCEvent * pEvent) {
    return true;
}

void FightScene::ccTouchEnded(CCTouch *touch, CCEvent * pEvent) {
    if (mAnimating || mCurPlayer != 0)
        return;
    
    // deaaaaaad!
    if (mFightData.mHealth[0] == 0 || mFightData.mHealth[1] == 0)
        return;
    
    CCPoint point = convertTouchToNodeSpace(touch);
    CCPoint coord = tileCoordForPoint(point);
    Tile* tile = getTile(coord.x, coord.y);
    
    if (!tile)
        return;
    
    if (mCurPickX == -1) {
        if (tile->type == Tile_Empty)
            return;
        
        // first pick
        mCurPickX = coord.x;
        mCurPickY = coord.y;
        
        //tile->sprite->setScale(2.0f);
        tile->sprite->runAction(CCScaleTo::actionWithDuration(0.15f, 3.0f));
    } else {
        // swap pick
        //tile->sprite->setScale(2.0f);
        if (tile->sprite)
            tile->sprite->runAction(CCScaleTo::actionWithDuration(0.15f, 3.0f));

        if (tile->type == Tile_Empty || !canSwap(mCurPickX, mCurPickY, coord.x, coord.y)) {
            // TODO: play a BZZZ NUH UH sound here
            if (tile->sprite)
                tile->sprite->runAction( CCScaleTo::actionWithDuration(0.25f, 2.0f) );
            if (getTile(mCurPickX, mCurPickY)->sprite)
                getTile(mCurPickX, mCurPickY)->sprite->runAction( CCScaleTo::actionWithDuration(0.25f, 2.0f) );
        } else {
            doSwap(mCurPickX, mCurPickY, coord.x, coord.y);
        }
        mCurPickX = -1;
    }
}

void FightScene::ccTouchCancelled(CCTouch *touch, CCEvent * pEvent) {
}

void FightScene::ccTouchMoved(CCTouch *touch, CCEvent * pEvent) {
}


CCScene* FightScene::scene(FightData data) {
    CCScene *scene = CCScene::node();
    FightScene* layer = FightScene::node();
    layer->mFightData = data;
    layer->setup();
    scene->addChild(layer);
    return scene;
}
