#pragma once

#include "cocos2d.h"
#include "GLESRender.h"
#include "SimpleAudioEngine.h"
#include "Random.h"

using namespace cocos2d;

struct FightData {
    bool bIsValid;
    CCPoint pos;
    int mHealth[2];
    int mMaxHealth[2];
    int mExperience[2];
    int mCash[2];
};

class FightScene : public CCLayer {
public:
    LAYER_NODE_FUNC(FightScene);
    virtual bool init();
    void setup();
    virtual void cleanup();
    static CCScene* scene(FightData data);

    void onBack();
    virtual bool ccTouchBegan(CCTouch *touch, CCEvent * pEvent);
    virtual void ccTouchEnded(CCTouch *touch, CCEvent * pEvent);
    virtual void ccTouchCancelled(CCTouch *touch, CCEvent * pEvent);
    virtual void ccTouchMoved(CCTouch *touch, CCEvent * pEvent);

private:
    CCSprite* mAvatar[2];
    CCSprite* mAvatarHealth[2];
    FightData mFightData;
    
    enum ETileType {
        Tile_Empty=-1,
        Tile_R=0,
        Tile_G,
        Tile_Cash,
        Tile_Attack,
        Tile_Experience,
        Tile_Health,
        Tile_NumTypes
    };
    
    typedef struct Tile {
        ETileType type;
        CCSprite* sprite;
        bool mark;  // if its a tile that will be destroyed (part of a 3+ chain)
    } Tile;
    
    CCSprite* spriteForTileType(ETileType type);
    ETileType randomTileType();
    
    void flashText(const char* str, int fontSize, const ccColor3B& color3, float travel, float hold);
    void flashTextCentered(const char* str, int fontSize, const ccColor3B& color3, float travel, float hold);
    
    void onGameWin(ccTime dt);
    void onGameLose(ccTime dt);
                
    CCPoint tileCoordForPoint(CCPoint p);
    CCPoint pointCoordForTile(int x, int y);
    bool swapHasAffect(Tile* A, Tile* B);
    bool canSwap(int x1, int y1, int x2, int y2);
    void doSwap(int x1, int y1, int x2, int y2);
    void onSwapComplete();
    bool canCollapse();
    bool canMakeMove();
    
    void onNoMoreMoves(ccTime dt);
    void onCollapseComplete();
    
    void updateHealthBars();
    void setupTiles();
    
    Tile* getTile(int x, int y);
    bool destroyMatches(bool bForceClearBoard=false);
    bool collapse();
    
    void clearMarks();
    bool markTiles();
    bool foundMatch();
    
    void beginTurn();
    void doAITurn();
    void checkEndGame();
    
    Tile* mTiles;
    int mTileSize;
    int mWidth, mHeight;
    Random mRandom;
    
    int mMatchSize;
    int mCurPickX, mCurPickY;
    int mAnimating;
    bool mbSkipEffects;
    
    int mCurPlayer;
};