#include "GameScene.h"
#include "CharScene.h"
#include "InitialScene.h"
#include "Character.h"

using namespace cocos2d;
using namespace CocosDenshion;

enum {
	kTagTileMap = 1,
};

bool GameScene::init() {
    if (!CCLayer::init())
        return false;
    
    setIsTouchEnabled( true );
    mIsWalking = false;
    
	CCSize screenSize = CCDirector::sharedDirector()->getWinSize();
    
    mMap = CCTMXTiledMap::tiledMapWithTMXFile("map0.tmx");
    mBackground = mMap->layerNamed("background");
    mForeground = mMap->layerNamed("foreground");
    mFights = mMap->layerNamed("fights");
    mFights->setIsVisible(false);
    mObjects = mMap->objectGroupNamed("objects");
    
	addChild(mMap, -1, kTagTileMap);
    
    CCSize s = mMap->getContentSize();
	
    
    CCSpriteFrameCache::sharedSpriteFrameCache()->addSpriteFramesWithFile("sprites.plist");
    CCSpriteBatchNode *spriteSheet = CCSpriteBatchNode::batchNodeWithFile("sprites.pvr.gz");
    spriteSheet->getTexture()->setAliasTexParameters(); // pixelate instead of blur
    addChild(spriteSheet);
    

    
    mPlayer = CCSprite::spriteWithSpriteFrameName("bunny2");
//    mPlayer->setScale(2.0f);
//    spriteSheet->addChild(mPlayer);
    
    mMap->addChild(mPlayer, mMap->getChildren()->count());


    // move to spawn loc
    CCStringToStringDictionary* dict = mObjects->objectNamed("spawn");
    int x = dict->objectForKey("x")->toInt();
    int y = dict->objectForKey("y")->toInt();
    mPlayer->setPosition(ccp(x, y));
    
    float cx = (screenSize.width - mMap->getMapSize().width*mMap->getTileSize().width)*0.5f;
    float cy = (screenSize.height - mMap->getMapSize().height*mMap->getTileSize().height)*0.5f;
    setPosition( ccp(cx,cy) );
    
    return true;
}

void GameScene::battleEnded(FightData data) {
    if (data.mHealth[0] > 0 && data.mHealth[1] <= 0) {
        // you won, raise the building
        mForeground->removeTileAt(ccp(data.pos.x,data.pos.y));
        
        Character* me = Character::sharedInstance();
        
        me->wins++;
        me->cash += data.mCash[0];
        me->experience += data.mExperience[0];
        me->level = 1 + me->experience / 5;
        
        // TODO: end game condition
    } else if (data.mHealth[0] <= 0 && data.mHealth[1] > 0) {
        Character::sharedInstance()->losses++;
    } else {
        Character::sharedInstance()->flees++;
    }
}

CCScene* GameScene::scene() {
    FightData data = {false};
    return GameScene::scene(data);
}
    
CCScene* GameScene::scene(FightData data) {
    
    static CCScene *scene = NULL;
    static GameScene *layer = NULL;
    
    if (!scene) {
        scene = CCScene::node();
        scene->retain(); // we keep this alive forevarr
    
        layer = GameScene::node();
        scene->addChild(layer);
        
        // menu
        CCLabelTTF *label;
        CCMenuItem *button, *button2;
        
        label = CCLabelTTF::labelWithString("[char]", "Futura-CondensedExtraBold", 32);
        label->setColor( ccc3(200,0,200) );
        button = CCMenuItemLabel::itemWithLabel(label, layer, menu_selector(GameScene::onChar));

        CCPoint pt = label->getPosition();
        pt.x += 100;
        
        label = CCLabelTTF::labelWithString("[quit]", "Futura-CondensedExtraBold", 32);
        label->setColor( ccc3(200,0,200) );
        CCSize screenSize = CCDirector::sharedDirector()->getWinSize();

        button2 = CCMenuItemLabel::itemWithLabel(label, layer, menu_selector(GameScene::onMenu));
        button2->setPosition( pt );
        
        CCMenu *menu = CCMenu::menuWithItems(button, button2, NULL);
        menu->setPosition(50, screenSize.height-25);
        scene->addChild(menu);
        
        // TODO: main menu button
    }
    
    
    if (data.bIsValid) {
        layer->battleEnded(data);
    }
    
    CocosDenshion::SimpleAudioEngine::sharedEngine()->playBackgroundMusic("wolfram.dance.1585.1191346988.mp3", true);
       
    return scene;
}

//
void GameScene::onChar() {
    CCActionManager::sharedManager()->removeAllActionsFromTarget(mPlayer);
    mIsWalking = false;
    
    CCTransitionScene* transition = CCTransitionSlideInR::transitionWithDuration(1.0, CharScene::scene());
    CCDirector::sharedDirector()->replaceScene(transition);
}

void GameScene::onMenu() {
    CCActionManager::sharedManager()->removeAllActionsFromTarget(mPlayer);
    mIsWalking = false;

    CCTransitionScene* transition = CCTransitionSlideInT::transitionWithDuration(1.0, InitialScene::scene());
    CCDirector::sharedDirector()->replaceScene(transition);
}

//
void GameScene::startBattle(int type, int level, CCPoint pos) {
    if (type == 1) {
        CocosDenshion::SimpleAudioEngine::sharedEngine()->playBackgroundMusic("wolfram.world.1585.2372048234.mp3", true);
    } else {
        CocosDenshion::SimpleAudioEngine::sharedEngine()->playBackgroundMusic("wolfram-1.world.1585.2372048234.mp3", true);
    }
    
    FightData data;
    data.pos = pos;
    data.mMaxHealth[0] = 25;
    data.mMaxHealth[1] = 25;
    data.mExperience[0] = data.mExperience[1] = 0;
    data.mCash[0] = data.mCash[1] = 0;
    
    CCTransitionScene* transition = CCTransitionRotoZoom::transitionWithDuration(1.0, FightScene::scene(data));
    CCDirector::sharedDirector()->replaceScene(transition);
}


void GameScene::centerViewport(CCPoint p) {
	CCSize screenSize = CCDirector::sharedDirector()->getWinSize();
    CCSize tileSize = mMap->getTileSize();
    int x = MAX(p.x, screenSize.width * 0.5f);
    int y = MAX(p.y, screenSize.height * 0.5f);
    x = MIN(x, (mMap->getMapSize().width * mMap->getTileSize().width) - screenSize.width*0.5f);
    y = MIN(y, (mMap->getMapSize().height * mMap->getTileSize().height) - screenSize.height*0.5f);
    CCPoint actualPosition = ccp(x, y);
    CCPoint centerOfView = ccp(screenSize.width*0.5f, screenSize.height*0.5f);
    CCPoint viewPoint = ccpSub(centerOfView, actualPosition);
    setPosition(viewPoint);
}


void GameScene::ccTouchesEnded(CCSet* touches, CCEvent* event) {
    if (mIsWalking)
        return;
    
    mWalkTo = convertTouchToNodeSpace((CCTouch*)touches->anyObject());

    CCPoint tp = tileCoordForPosition(mWalkTo);
    if (tp.x < 0 || tp.y < 0 || tp.x >= mMap->getMapSize().width || tp.y >= mMap->getMapSize().height)
        return;

    mIsWalking = true;
    
    CocosDenshion::SimpleAudioEngine::sharedEngine()->playEffect("5hop.wav");
    CCFiniteTimeAction* move = CCJumpTo::actionWithDuration(2.0f, mWalkTo, 30, 5);
    CCFiniteTimeAction* complete = CCCallFunc::actionWithTarget(this, callfunc_selector(GameScene::moveComplete));
    mPlayer->runAction( CCSequence::actionOneTwo(move, complete) );
}

CCPoint GameScene::tileCoordForPosition(CCPoint p) {
    CCSize tileSize = mMap->getTileSize();
    CCSize mapSize = mMap->getMapSize();
    int tx = p.x / tileSize.width;
    int ty = ((mapSize.height * tileSize.height) - p.y) / tileSize.height;
    return ccp(tx, ty);
}

int GameScene::buildingAtPosition(CCPoint p) {
    CCPoint tp = tileCoordForPosition(p);
    int gid = mForeground->tileGIDAt(tp);
    CCStringToStringDictionary* dict = mMap->propertiesForGID(gid);
    if (dict) {
        CCString* building = dict->objectForKey("building");
        if (building) {
            return building->toInt();
        }
    }
    
    return 0;
}

int GameScene::levelAtPosition(CCPoint p) {
    CCPoint tp = tileCoordForPosition(p);
    int gid = mFights->tileGIDAt(tp);
    CCStringToStringDictionary* dict = mMap->propertiesForGID(gid);
    if (dict) {
        CCString* str = dict->objectForKey("level");
        if (str) {
            return str->toInt();
        }
    }
    
    return 0;
}

void GameScene::moveComplete() {
    mIsWalking = false;
    
    CCPoint p = mPlayer->getPosition();
//    centerViewport(mPlayer->getPosition());
    
    int building = buildingAtPosition(p);
    if (building) {
        int level = levelAtPosition(p);
        
        if (level) {
            startBattle(building, level, tileCoordForPosition(p));
        }
    }
}

//http://codewars.net/304/layer-clipping-in-cocos2d-x/
//void GameScene::visit()
//{
//    glPushMatrix();
//    
//    glEnable(GL_SCISSOR_TEST);
//    
//    CCEGLView::sharedOpenGLView().setScissorInPoints(
//                                                     clippingRegion.origin.x + getPosition().x, clippingRegion.origin.y + getPosition().y,
//                                                     clippingRegion.size.width, clippingRegion.size.height);
//    
//    CCNode::visit();
//    glDisable(GL_SCISSOR_TEST);
//    glPopMatrix();
//}
